<?php

/*
Plugin Name: Photopress Plugin
Plugin URI: http://isaac.wedin.org/photopress/
Description: Adds buttons to the quicktags toolbar to launch popup tools that allow users to upload photos and browse previously-uploaded photos, then insert linked-thumbnail code into posts. As a huge bonus, there's also a random image function AND a simple photo album. Extensive options can be configured in the Options:Photopress admin page, and uploaded images can be categorized at Manage:Photopress Album.
Version: 0.4
Author: Isaac Wedin
Author URI: http://isaac.wedin.org/
*/

/*  Copyright 2004  Isaac Wedin (email : isaac@wedin.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

include_once('photopress/photopress-inc.php');

/* this function inserts javascript in the admin header to open the Upload and Browse popups */
function pp_popup_js() {
	echo '
	<script language="JavaScript" type="text/javascript"><!--
	/* it should be possible to do this with a single function that accepts an href, but that breaks the later stuff that calls these functions... */
	function pp_upload_popup() {
		var mylink = "' . get_settings('siteurl') . '/wp-content/plugins/photopress/upload.php";
		var windowname = "upload";
		if (! window.focus) return true;
			var href;
		if (typeof(mylink) == "string")
			href = mylink;
		else
			href = mylink.href;
		window.open(href, windowname, "width=400,height=600,scrollbars=yes");
		return false;
	}

	function pp_browse_popup() {
                var mylink = "' . get_settings('siteurl') . '/wp-content/plugins/photopress/browse.php";
                var windowname = "browse";
                if (! window.focus) return true;
                        var href;
                if (typeof(mylink) == "string")
                        href = mylink;
                else
                        href = mylink.href;
                window.open(href, windowname, "width=400,height=600,scrollbars=yes");
                return false;
        }
	//--></script>
	';
}

add_action('admin_head','pp_popup_js');

/*This function adds the Upload and Browse buttons to the quicktags menu. Taken from ContactForm, which took it from Alex King's JS Quicktags.*/
function pp_insert_buttons() {
	echo '
	<script language="JavaScript" type="text/javascript">
	<!--
	document.getElementById("quicktags").innerHTML += "<input type=\"button\" class=\"ed_button\" id=\"ed_upload\" value=\"Browse\" onclick=\"return pp_browse_popup();\" /><input type=\"button\" class=\"ed_button\" id=\"ed_upload\" value=\"Upload\" onclick=\"return pp_upload_popup();\" />";
	//-->
	</script>
	';
}

add_action('admin_footer', 'pp_insert_buttons');

/* random image function, returns a random linked thumb by default. */
/* set $type to 2 to get a bare tagged thumb, 3 for just the image name */
function pp_random_image($image_array, $type = 1) {
	global $pp_options, $pp_imagelink, $pp_photosaddress, $pp_photospath, $pp_target;
	sort($image_array);
	$image_count = sizeof($image_array);
	if ($image_count > 0) {
		if ($image_count > 1) {
			$r = rand(0,($image_count-1));
			$random_image = $image_array[$r];
		} else {
			$random_image = $image_array[0];
		}
		$random_image_thumb = $pp_options['pp_thumbprefix'].$random_image;
		$random_image_thumb_size = getimagesize("$pp_photospath/$random_image_thumb");
		$random_alt_text = str_replace("_", " ", substr($random_image, 0, -4));
		$random_thumb = '<img class="centered" src="' . $pp_photosaddress . '/' . $random_image_thumb . '" ' . $random_image_thumb_size[3] . ' alt="' . $random_alt_text . '" />';
		$random_linked_thumb =  '<a href="' . $pp_imagelink . $random_image . '" title="' . $random_alt_text . '" target="' . $pp_target . '">' . $random_thumb . '</a>';
	} else {
		$random_linked_thumb = 'no images yet';
	}
	if ($type == 3) {
		return $random_image;
	} elseif ($type == 2) {
		return $random_thumb;
	} else {
		return $random_linked_thumb;
	}
}

function pp_random_image_bare() {
	$full_array = array();
	$full_array = pp_folder_contents();
	echo pp_random_image($full_array);
}

/* put a random link to the photo album and/or a random thumbnail in the meta section of the sidebar, depending on the options */

function pp_stuff_in_meta() {
	global $pp_options, $pp_albumaddress;
	if (get_option('pp_options')) {
		$pp_options = get_option('pp_options');
	}
	$image_array = pp_folder_contents();
	$stuff_for_meta = '';
	if ($pp_options['pp_meta_link'] == 1) {
		$stuff_for_meta .= '<li><a href="' . $pp_albumaddress . '">Photo Album</a></li>' . "\n";
	}
	if ($pp_options['pp_meta_rand'] == 1) {
		$stuff_for_meta .= '<li>' . pp_random_image($image_array) . "</li>\n";
	}
	echo $stuff_for_meta;
}

add_action('wp_meta', 'pp_stuff_in_meta');

function pp_album() {
	global $pp_albumaddress;
	echo '<h2><a href="' . $pp_albumaddress . '">Photo Album</a></h2>';
	if (isset($_GET['pp_image'])) {
		$image = $_GET['pp_image'];
		pp_display_image($image);
	} elseif (isset($_GET['pp_cat'])) {
		$cat = $_GET['pp_cat'];
		pp_display_cat($cat);
	} else {
		pp_display_main();
	}
}

// returns an array of image names within $pp_photospath
function pp_folder_contents() {
	global $pp_photospath, $pp_thumbprefix, $pp_allowedtypes;
	$allowedtypes = trim(strtolower($pp_allowedtypes));
	$allowedtypes = preg_replace("/ /","|",$allowedtypes);
	$fileglob = '@\.(' . $allowedtypes . ')$@i';
	$handle = opendir($pp_photospath);
	$list_array = array();
	while (false !== ($folder_contents = readdir($handle))) {
		if ((preg_match($fileglob, $folder_contents)) && (strstr($folder_contents, $pp_thumbprefix))) {
			$image_name = substr($folder_contents, strlen($pp_thumbprefix));
			$list_array[$image_name] = $image_name;
		}
	}
	sort($list_array);
	return $list_array;
}

// displays images from $category
function pp_display_cat($category, $pp_embed = 0) {
	global $pp_albumaddress, $pp_photosaddress, $pp_albumpostfix, $pp_thumbprefix, $pp_album_columns, $pp_albumpostfix, $pp_allowedtypes, $pp_imagelink, $pp_photospath;
	if ($pp_embed == 0 ) {
		echo '<h3>' . stripslashes($category) . '</h3>';
	}
	$column = 1;
	$cat_images = array();
	$cat_images = pp_cat_images($category);
	echo '<table id="pp_gallery">';
	foreach($cat_images as $array_img) {
		if($column == 1) {
			echo '<tr>';
		}
		$thumbsize = getimagesize("$pp_photospath/$pp_thumbprefix$array_img");
		$cleanedname = strtr(substr($array_img, 0, -4), "_", " ");
		echo '<td class="pp_cell"><a href="' . $pp_imagelink . $array_img . '" title="' . $cleanedname . '"><img src="' . $pp_photosaddress . '/' . $pp_thumbprefix . $array_img . '" ' . $thumbsize[3] . ' class="pp_centered" alt="' . $cleanedname . '" /><br />' . $cleanedname . '</a></td>';
		if($column == $pp_album_columns) {
			echo '</tr>';
			$column = 0;
		}
		$column++;
	}
	echo '</table>';
}

function pp_cat_images($category) {
	$category_array = array();
	if (get_option('pp_album_cats')) {
		$category_array = get_option('pp_album_cats');
		$category_array = array_filter($category_array, 'pp_not_empty');
	}
	if($category == 'Default') {
		$full_array = pp_folder_contents();
		if (empty($category_array)) {
			return $full_array;
		}
		$cat_array = array_keys($category_array);
		$i = 0;
		foreach($cat_array as $image) {
			$cat_images[$i] = substr_replace($image, '.', -4, 1);
			$i++;
		}
		$no_cat = array_diff($full_array, $cat_images);
		return $no_cat;
	}
	$cat_images = array();
	$i = 0;
	foreach($category_array as $array_img => $array_cat) {
		if($array_cat == $category) {
			$cat_images[$i] = substr_replace($array_img, '.', -4, 1);
			$i++;
		}
	}
	return $cat_images;
}

// returns an image's category, defaults to 'Default' if none
function pp_image_cat($image) {
	$category_array = array();
	if (get_option('pp_album_cats')) {
		$category_array = get_option('pp_album_cats');
		$category_array = array_filter($category_array, 'pp_not_empty');
	}
	$category = 'Default';
	foreach($category_array as $img => $cat) {
		if($image == substr_replace($img, '.', -4, 1)) {
			$category = $cat;
		}
	}
	return $category;
}

function pp_display_main() {
	global $pp_albumaddress, $pp_catpostfix, $pp_album_columns;
	$categories = array();
	$categories = pp_list_cats();
	$k = 1;
	echo '<table id="pp_gallery">';
	if (count($categories) > 1) {
		foreach($categories as $cat) {
			if ($k == 1) {
				echo '<tr>';
			}
			$cat_images = pp_cat_images($cat);
			$image_count = count($cat_images);
			echo '<td class="pp_cell"><a href="' . $pp_albumaddress . $pp_catpostfix . stripslashes($cat) . '" title="' . stripslashes($cat) . '">' . pp_random_image($cat_images,2) . '<br />' . stripslashes($cat) . ' (' . $image_count . ')</a></td>';
			if ($k == $pp_album_columns) {
				echo '</tr>';
				$k = 0;
			}
			$k++;
		}
	} elseif (count($categories) == 1) {
		echo '<tr>';
		$cat_images = pp_cat_images($categories[0]);
		$image_count = count($cat_images);
		echo '<td class="pp_cell"><a href="' . $pp_albumaddress . $pp_catpostfix . $categories[0] . '" title="' . stripslashes($categories[0]) . '">' . pp_random_image($cat_images,2) . '<br />' . stripslashes($categories[0]) . ' (' . $image_count . ')</a></td>';
		echo '</tr>';
	} else {
		echo '<tr><td><strong>No photos uploaded yet</strong></td></tr>';
	}
	echo '</table>';
}

function pp_not_empty($string) {
	return !empty($string);
}
	
// returns an array of category names by finding unique cats in clean cat list
function pp_list_cats() {
	$category_array = array();
	if (get_option('pp_album_cats')) {
		$category_array = get_option('pp_album_cats');
		$category_array = array_filter($category_array, 'pp_not_empty');
	}
	if(pp_cat_images('Default')) {
		$category_array[] = 'Default';
	}
	if (!empty($category_array)) {
		$category_array = array_unique($category_array);
		sort($category_array);
	}
	return $category_array;
}

function pp_allowed($image) {
	global $pp_allowedtypes;
	$allowedtypes = trim(strtolower($pp_allowedtypes));
	$allowedtypes = preg_replace("/ /","|",$allowedtypes);
	$image = strtolower($image);
	$fileglob = '@\_(' . $allowedtypes . ')$@i';
	return(preg_match($fileglob, $image));
}

// displays an image, linked to a larger version) 
function pp_display_image($image) {
	global $pp_albumaddress, $pp_photosaddress, $pp_albumpostfix, $pp_catpostfix, $pp_photospath;
	$image_cat = pp_image_cat($image);
	$cleanedname = strtr(substr($image, 0, -4), "_", " ");
	echo '<div id="pp_meta"><strong><a href="' . $pp_albumaddress . $pp_catpostfix . stripslashes($image_cat) . '">' . stripslashes($image_cat) . '</a> : ' . $cleanedname . '</strong></div>' . "\n";
	$cat_array = pp_cat_images($image_cat);
	$image_size = getimagesize("$pp_photospath/$image");
	sort($cat_array);
	$itemcount = sizeof($cat_array);
	if ($itemcount > 1) {
	$prevnext = '<div id="pp_prevnext">' . "\n";
	for ($j = 0; $j < $itemcount; $j++) {
		$item = $cat_array[$j];
		if ($image == $item) {
			if ($cat_array[$j-1]) {
				$prevnext .= '<a class="pp_prev" href="' . $pp_albumaddress . $pp_albumpostfix . $cat_array[$j-1] . '">Previous</a>' . "\n";
			} else {
				$prevnext .= '<div class="pp_prev"></div>' . "\n";
			}
			if ($cat_array[$j+1]) {
				$prevnext .= '<a class="pp_next" href="' . $pp_albumaddress . $pp_albumpostfix . $cat_array[$j+1] . '">Next</a>' . "\n";
			} else {
				$prevnext .= '<div class="pp_next"></div>' . "\n";
			}
		}
	}
	$prevnext .= "</div>\n";
	echo $prevnext;
	}
	echo '<div id="pp_gallery">';
	echo '<a href="' . $pp_photosaddress . '/' . $image . '" title="' . $cleanedname . '" target="_blank"><img class="pp_centered" src="' . $pp_photosaddress . '/' . $image . '" ' . $image_size[3] . ' alt="' . $cleanedname . '" /></a>';
	echo "</div>\n";
}

function pp_album_template() {
	if ($_GET['pp_album']) {
	include('wp-content/plugins/photopress/album.php');
	exit;
	}
	return;
}

add_action('template_redirect','pp_album_template');

function pp_add_options_page() {
	add_options_page('Photopress', 'Photopress', 8, __FILE__, 'pp_options_subpanel');
}

function pp_options_subpanel() {
	if (get_option('pp_options')) {
		$pp_options = get_option('pp_options');
	} else {
		global $pp_options;
	}
	if (isset($_POST['pp_options_update'])) {
		echo '<div class="updated">Photopress options updated.</div>';
	}
	echo '
	<div class="wrap">
	<h2>Photopress Options</h2>
	<form name="pp_options" method="post">
	<input type="hidden" name="pp_options_update" value="update" />
	<fieldset class="options">
	<table width="100%" cellspacing="2" cellpadding="5" class="editform">
	<tr>
	<th scope="row">Maximum size: </th>
	<td><input name="pp_maxk" type="text" id="pp_maxk" value="' . $pp_options['pp_maxk'] . '" size="4" /> 
	Kilobytes (KB)</td>
	</tr>
        <tr>
        <th scope="row">Maximum resized image size: </th>
        <td><input name="pp_maxsize" type="text" id="pp_maxsize" value="' . $pp_options['pp_maxsize'] . '" size="4" />
        pixels</td>
        </tr>
        <tr>
        <th scope="row">Maximum thumbnail size: </th>
        <td><input name="pp_thumbsize" type="text" id="pp_thumbsize" value="' . $pp_options['pp_thumbsize'] . '" size="4" />
        pixels</td>
        </tr>
        <tr>
        <th scope="row">Quality for resized JPEGs: </th>
        <td><input name="pp_quality" type="text" id="pp_quality" value="' . $pp_options['pp_quality'] . '" size="3" /><br />
        Recommended: between 70 and 95</td>
        </tr>
        <tr>
        <th scope="row">Photos folder: </th>
        <td><input name="pp_photofolder" type="text" id="pp_photofolder" value="' . $pp_options['pp_photofolder'] . '" size="10" /><br />
        Folder in wp-content to place images in, should be writable.</td>
        </tr>
        <tr>
        <th scope="row">Thumb prefix: </th>
        <td><input name="pp_thumbprefix" type="text" id="pp_thumbprefix" value="' . $pp_options['pp_thumbprefix'] . '" size="10" /><br />
        String to add to created thumbnails.</td>
        </tr>
	<tr>
	<th scope = "row">Album name: </th>
        <td><input name="pp_albumname" type="text" id="pp_albumname" value="' . $pp_options['pp_albumname'] . '" size="10" /><br />
	Default should be "?pp_album=1" - use something like "album" for permalinks.</td>
	</tr>
	<th scope = "row">Album postfix: </th>
        <td><input name="pp_albumpostfix" type="text" id="pp_albumpostfix" value="' . $pp_options['pp_albumpostfix'] . '" size="10" /><br />
	Default should be "&pp_image=" - use something like "/" for permalinks.</td>
	</tr>
	<tr>
	<th scope = "row">Category postfix: </th>
        <td><input name="pp_catpostfix" type="text" id="pp_catpostfix" value="' . $pp_options['pp_catpostfix'] . '" size="10" /><br />
	Default should be "&pp_cat=" - use something like "/category/" for permalinks.</td>
	</tr>
	<tr>
	<th scope="row">Insert Thumbs:</th>
	<td>
	<label><input name="pp_insert_thumb" type="radio" value="1" ';
	if ($pp_options['pp_insert_thumb'] == '1') {echo 'checked="checked" ';}
	echo '/> Yes</label><br />
	<label><input name="pp_insert_thumb" type="radio" value="0" ';
	if ($pp_options['pp_insert_thumb'] == '0') {echo 'checked="checked" ';}
	echo ' /> No</label>
	</td>
	</tr>
        <tr>
        <th scope="row">Link to Album:</th>
        <td>
	<label><input name="pp_album" type="radio" value="1" ';
	if ($pp_options['pp_album'] == '1') {echo 'checked="checked" ';}
	echo '/> Yes</label><br />
	<label><input name="pp_album" type="radio" value="0" ';
	if ($pp_options['pp_album'] == '0') {echo 'checked="checked" ';}
	echo '/> No</label><br />
        </td>
        </tr>
        <tr>
        <th scope="row">Insert Link in Meta:</th>
	<td>
	<label><input name="pp_meta_link" type="radio" value="1" ';
	if ($pp_options['pp_meta_link'] == '1') {echo 'checked="checked" ';}
	echo '/> Yes</label><br />
	<label><input name="pp_meta_link" type="radio" value="0" ';
	if ($pp_options['pp_meta_link'] == '0') {echo 'checked="checked" ';}
	echo '/> No</label><br />
        </td>
        </tr>
        <tr>
        <th scope="row">Insert Random Image in Meta:</th>
	<td>
	<label><input name="pp_meta_rand" type="radio" value="1" ';
	if ($pp_options['pp_meta_rand'] == '1') {echo 'checked="checked" ';}
	echo '/> Yes</label><br />
	<label><input name="pp_meta_rand" type="radio" value="0" ';
	if ($pp_options['pp_meta_rand'] == '0') {echo 'checked="checked" ';}
	echo '/> No</label><br />
        </td>
        </tr>
        <tr>
	<th scope="row">Add Failsafe Buttons:</th>
	<td>
	<label><input name="pp_failsafe_buttons" type="radio" value="1" ';
	if ($pp_options['pp_failsafe_buttons'] == '1') {echo 'checked="checked" ';}
	echo '/> Yes</label><br />
	<label><input name="pp_failsafe_buttons" type="radio" value="0" ';
	if ($pp_options['pp_failsafe_buttons'] == '0') {echo 'checked="checked" ';}
	echo '/> No</label><br />
	</td>
	</tr>
        <tr>
        <th scope="row">Album columns: </th>
        <td><input name="pp_album_columns" type="text" id="pp_album_columns" value="' . $pp_options['pp_album_columns'] . '" size="3" /><br />
	Probably 2 if you have a sidebar, maybe 3 otherwise.
	</td>
        </tr>
	<tr>
	<th valign="top" scope="row">Allowed file extensions:</th>
	<td><input name="pp_allowedtypes" type="text" id="pp_allowedtypes" value="' . $pp_options['pp_allowedtypes'] . '" size="40" />
	<br />
	Recommended: <code>jpg png gif</code> (because that\'s all that\'ll get resized anyway)
	</td>
	</tr>
        <tr>
        <th scope="row">Class for inserted image style: </th>
        <td><input name="pp_align" type="text" id="pp_align" value="' . $pp_options['pp_align'] . '" size="10" /><br />
        (alignleft, alignright, and centered are available in the default theme - add a new class to your theme and enter the name here for custom style)
	</td>
        </tr>
	<tr>
	<th scope="row">Minimum level to upload and browse:</th>
	<td><select name="pp_min_level" id="pp_min_level">';
	for ($i = 1; $i <= 10; $i++) {
		echo '
		<option value="' . $i . '"';
		if ($pp_options['pp_min_level'] == $i) { echo ' selected="selected"'; }
		echo '>' . $i . '</option>';
	}
	echo '
	</select>
	</td>
	</tr>
	</table> 
	</fieldset>

	<p class="submit">
		<input type="submit" name="Submit" value="Update Options &raquo;" />
	</p>
</form> 
</div>
';
}

add_action('admin_menu', 'pp_add_options_page');

if (isset($_POST['pp_options_update'])) {
	$pp_updated_options = array();
	$pp_updated_options = $_POST;
	add_option('pp_options');
	update_option('pp_options', $pp_updated_options);
}

function pp_album_management() {
	global $pp_imagelink, $pp_thumbprefix, $pp_photosaddress, $pp_target;
	if (isset($_POST['pp_album_update'])) {
		echo '<div class="updated">Photopress album categories updated.</div>';
	}
	if (get_option('pp_album_cats')) {
		$pp_album_cats = array();
		$pp_album_cats = get_option('pp_album_cats');
	}
	$pp_album_array = array();
	$pp_album_array = pp_folder_contents();
	sort($pp_album_array);
	echo '
	<div class="wrap">
	<h2>Photopress Album Categories</h2>
	<form name="pp_album_cats" method="post">
	<input type="hidden" name="pp_album_update" value="update" />
        <fieldset class="options">
	<table width="100%" cellspacing="2" cellpadding="5" class="editform">
	';
	foreach($pp_album_array as $pp_image_name) {
		$pp_image_name_underscored = preg_replace("/\./","_",$pp_image_name);
		echo '<tr><th scope="row"><img src="' . $pp_photosaddress . '/' . $pp_thumbprefix . $pp_image_name . '" /><br />';
		echo $pp_image_name . '</th>';
		echo '<td><input name="' . $pp_image_name . '" type="text" id="' . $pp_image_name . '" value="' . stripslashes($pp_album_cats[$pp_image_name_underscored]) . '" size="10" /></td></tr>
		';
	}
	echo '
	</table>
	</fieldset>
	<p class="submit">
	<input type="submit" name="Submit" value="Update Categories &raquo;" />
	</p>
</form>
</div>';
}

function pp_add_management_page() {
	global $pp_min_level;
	add_management_page('Photopress Album', 'Photopress Album', $pp_min_level, __FILE__, 'pp_album_management');
}

add_action('admin_menu', 'pp_add_management_page');

if (isset($_POST['pp_album_update'])) {
	$pp_updated_cats = array();
	$pp_updated_cats = $_POST;
	foreach($pp_updated_cats as $img => $cat) {
		if(!pp_allowed($img)) {
			unset($pp_updated_cats[$img]);
		}
	}
	add_option('pp_album_cats');
	update_option('pp_album_cats', $pp_updated_cats);
}


if ($pp_failsafe_buttons == '1') {
	function pp_failsafe_buttons() {
		echo '
			<input type="button" value="Upload" onclick="return pp_upload_popup()" /> <input type="button" value="Browse" onclick="return pp_browse_popup()" />';
	}
	add_action('simple_edit_form', 'pp_failsafe_buttons');
	add_action('edit_form_advanced', 'pp_failsafe_buttons');
}

?>
