<?php

/* Photopress browser, based on ImageBrowser plugin by Florian Jung */

require_once( dirname(dirname(dirname(dirname( __FILE__ )))) . "/wp-config.php");
include_once( "photopress-inc.php" );
get_currentuserinfo();
if ($user_level == 0) //Checks to see if user has logged in
        die (__("Try logging in"));
if ($user_level < $pp_min_level) //Checks to see if user's level is high enough
        die (__("Ask the administrator to promote you."));

// ====( configuration options )================================================
$ib_images_per_page = 10;  // display ... images per page in the browser
$ib_set_title = FALSE;     // set 'title'-attribute for images

// ====( STOP EDITING HERE )====================================================
$blogurl = get_settings('siteurl');

$title = 'Select Image';
$standalone = 1;

// ====( print header )=========================================================
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>WordPress &rsaquo; <?php 
bloginfo('name') ?> &rsaquo; <?php echo $title; ?></title>
<link rel="stylesheet" href="<?php echo $blogurl; ?>/wp-admin/wp-admin.css" type="text/css" />
<link rel="shortcut icon" href="<?php echo $blogurl; ?>/wp-images/wp-favicon.png" />
<meta http-equiv="Content-Type" content="text/html; charset=<?php 
	echo get_settings('blog_charset'); ?>" />
<script type="text/javascript">
<!-- // idocs.com's popup tutorial rules !
function targetopener(blah, closeme, closeonly) {
        if (! (window.focus && window.opener))return true;
        window.opener.focus();
        if (! closeonly)window.opener.document.post.content.value += blah;
        if (closeme)window.close();
        return false;
}
//-->
</script>

<style type="text/css">
img { border: 0px; }
#ibwrapper { margin: 8px; padding: 0; }
#step { margin 0; padding: 8px; background: #f0f8ff; border-bottom: 1px solid #69c; }
.image { margin: 0 0 12px 0; padding: 4px; 
	 background: #eeeeee; border: 1px solid #cccccc; 
}
.image .title { font-size: 1em; padding:0; margin: 0 0 8px 0; }
.image .highlight { color: #ff0000; 
}
.infoform { margin: 6px 0 12px 0; text-align: left; }
.select { margin: 6px 0 0 0; text-align: left; }
</style>
</head>
<body>
<?php

// ====( functions )============================================================

/**
 * get some useful infos for a file 
 *
 * @param  string $filename name of the file with path on disk (not url)
 * @return mixed FALSE if not found, else $meta for html and code creation
 */
function ib_get_fileinfo($file) {
    if (!is_file($file)) {
	return FALSE;
    }
    
    $meta = array();
    $meta['name'] = basename($file);

    // filesize
    if (FALSE !== ($temp = @filesize($file))) {
	$meta['filesize'] = $temp;
    }
    
    // filemtime (last modified)
    if (FALSE !== ($temp = @filemtime($file))) {
	$meta['mtime'] = $temp;
    }

    // image size (if possible)
    if (FALSE !== ($temp = @getimagesize($file))) {
	$meta['width'] = $temp[0];
	$meta['height'] = $temp[1];
	$meta['img'] = $temp[3];
    }
    
    return $meta;
} // ib_get_fileinfo


/** 
 * recognize the thumbnails by filename
 *
 * The thumbnails have the same names like the fullsize version
 * with a 'thumb_' added as prefix. Data of recognized thumbs is
 * added to the $meta of the fullsized image as $meta['thumb']
 *
 * @param array $files hash of filenames with metadata
 * @return array thumb-data is now under $files[$fullsize]['thumb']
 */
function ib_recognize_thumbs($files) {
    global $pp_thumbprefix;
    
    foreach ($files as $file => $meta) {
	if (0 !== strpos($file, $pp_thumbprefix)) 
	    continue; // not starting with thumbprefix

	// Check for a fullsize version
	$fullsize = substr($file,strlen($pp_thumbprefix));	
	if(isset($files[$fullsize])) {
	    // it actually *is* a thumb, so delete it from $files
	    unset($files[$file]); 
	    // ... add meta as 'thumb' to the fullsize version
	    $files[$fullsize]['thumb'] = $meta;
	    // ... and set mtime to the newest of both
	    if ($files[$fullsize]['mtime'] < $meta['mtime'])
		$files[$fullsize]['mtime'] = $meta['mtime'];
	}
    }
    
    return $files;
} // end ib_recognize_thumbs


/**
 * return files with allowed extension from upload directory
 *
 * The filenames are matched against the allowed file extension
 * from upload option. Thumbnails will be recognized
 * The returned array is 
 *
 * @return mixed hash with filename as key or FALSE
 */
function ib_get_uploaded_files() {
    global $pp_photospath, $pp_allowedtypes;
    $allowedtypes = trim(strtolower($pp_allowedtypes));
    $allowedtypes = preg_replace("/ /","|",$allowedtypes);
    $pathtophotos = $pp_photospath;
    $files = array();
    
    // regex for allowed filetypes, e.g. @\.(gif|png|jpg|jpeg)$@i
    $fileglob = '@\.(' . $allowedtypes . ')$@i';
    
    // open upload directory
    if (FALSE === ($upload_dir = opendir($pathtophotos))) {
	print "<p>Could no open upload  directory :-(</p>\n";
	return FALSE;
    }
    
    // sort out the allowed filetypes
    while (FALSE !== ($file = readdir($upload_dir))) {
	$file_on_disk = $pathtophotos . "/$file";
	
	if (!(is_file($file_on_disk) and preg_match($fileglob, $file))) {
	    // not a regular file or disallowed filetype
	    continue;
	}
	
	$files["$file"] = ib_get_fileinfo($file_on_disk);
    }
    
    // tidy up and return
    closedir($upload_dir);
    $files = ib_recognize_thumbs($files);
    return $files;
} // ib_get_uploaded_files()


/**
 * sort an array by the values of $array[$primary_key][$secondary_key]
 *
 * The function make use of a helper array for sorting.
 *
 * @param array $hash 2D hash to sort
 * @param string $secondary_key
 * @param string $sort_order should be one of 'asc', 'desc'
 * @return mixed FALSE if $secondary_key is not found, else the sorted array
 */
function ib_sort_hash_by_secondary_value($hash, $secondary_key,
					 $sort_order = 'asc') {
    $result = array();
    
    foreach ($hash as $key => $value) {
	if (!isset($value[$secondary_key])) return FALSE;
	$result[$key] = $value[$secondary_key];
    }
    // $helper is now: array ( primary_key => secondary_VALUE, ... )
    // just sort this array by the values
    if ('asc' == $sort_order) {
	asort($result);
    } else {
	arsort($result);
    }
    
    // insert the data into the array
    foreach ($result as $key => $value) {
	$result[$key] = $hash[$key];
    }
    
    return $result;
} // ib_sort_hash_by_secondary_key


/**
 * make <img ... /> html snippet for an image
 *
 * @see ib_make_code_simple_image()
 * @param  array  $image Metainfo for the image (includes name)
 * @param  mixed  $link FALSE (no link) or Metainfo
 * @return string html snippet
 */
function ib_make_html_img($image, $link=FALSE) {
    global $pp_photosaddress, $pp_align;

    $html = '';

    // ---- image code
    if (FALSE !== $image) {
	$html .= '<img class="' . $pp_align . '" src="' . $pp_photosaddress . "/"
	    . urlencode($image['name']) . '" ';
	if (0 !== $image['img']) // the getimagesize-html-snippet
	    $html .= $image['img'] . ' ';
	// add alternate text (if not set, then set to an empty string
	if (!isset($image['alt']))
   $image['alt'] = strtr(substr($image['name'], 0, -4), "_", " ");
 	$html .= 'alt="' . $image['alt'] . '" ';
	// add title (only if given)
	if (isset($image['title']))
	    $html .= 'title="' . $image['title'] . '" ';
 	$html .= '/>';
    } elseif (FALSE !== $link and isset($link['name'])) {
	// this will be the link text, if no image is given
	$html .= $link['name'];
    }
    
    // ---- $link code
    if (FALSE !== $link and isset($link['name'])) {
	// add it around the already existing code
	if (!isset($link['title']))
	    $link['title'] = '';
	$html = '<a href="' . $pp_photosaddress
	    . urlencode($link['name']) . '"'
	    . ' title="' . $link['title'] . '">'
	    . $html . '</a>';
    }

    return $html;

} // ib_make_html_single_image


/**
 * make code snippet for an image
 *
 * If $image is given, the function makes an '<img ... />' snippet
 * Setting $link makes an '<a ... />' snippet around the image.
 * If you set one of them to FALSE, the according snippet will
 * be omitted (you will get only an image or only the link).
 * If $image is FALSE but $link given, $link['name'] is used
 * as the text for the link and the Description as link title.
 *
 * @see ib_make_html_simple_image()
 * @param  array  $image FALSE or metainfo for the image
 * @param  mixed  $link FALSE or metainfo for the link
 * @return string code snippet with html-chars escaped
 */
function ib_make_code_single_image($image, $link = FALSE) {
    global $pp_photosaddress, $pp_align, $pp_imagelink, $pp_target, $pp_thumbprefix;

    $html = '';

    // ---- image code
    if (FALSE !== $image) {
	$html .= "&lt;img class=&quot;" . $pp_align . "&quot; src=&quot;" . $pp_photosaddress
	    . "/" . urlencode($image['name']) . '&quot; ';
	if (0 !== $image['img']) // the getimagesize-html-snippet
	    $html .= htmlspecialchars($image['img']) . ' ';
	// add alternate text (if not set, then set to an empty string
	if (!isset($image['alt'])) {
		if (substr($image['name'], 0, strlen($pp_thumbprefix)) == $pp_thumbprefix) {
     		$image['alt'] = strtr(substr($image['name'], strlen($pp_thumbprefix), -4), "_", " ");
	  	} else {
     		$image['alt'] = strtr(substr($image['name'], 0, -4), "_", " ");
	   }
	}
	$html .= "alt=&quot;{$image['alt']}&quot; ";
	// add title (only if given)
	if (isset($image['title']))
	    $html .= "title=&quot;{$image['title']}&quot; ";
	$html .= '/&gt;';
    } elseif (FALSE !== $link and isset($link['name'])) {
	// this will be the link text, if no image is given
	$html .= ib_make_code_single_image($link['thumb'],FALSE);
    }

    // ---- $link code
    if (FALSE !== $link and isset($link['name'])) {
	// add it around the already existing code
	if (!isset($link['title']))
       $link['title'] = strtr(substr($link['name'], 0, -4), "_", " ");
	$html = "&lt;a href=&quot;" . $pp_imagelink
	    . urlencode($link['name']) . "&quot; target=&quot;"
	    . $pp_target . "&quot; title=&quot;{$link['title']}&quot;&gt;"
	    . $html . '&lt;/a&gt;';
    }

    return $html;
} // ib_make_code_single_image


/**
 * remove magic Quotes
 *
 * @param array &$array POST or GET with magic quotes
 */
function ib_magic_quotes_remove(&$array) {
    if(!get_magic_quotes_gpc())
	return;
    foreach($array as $key => $elem) {
	if(is_array($elem))
	    ib_magic_quotes_remove($elem);
	else
	    $array[$key] = stripslashes($elem);
    }
} // ib_magic_quotes_remove

/**
 * make html for select element
 *
 * The hash $options should contain values and Description:
 * array( 
 *   'value' => array( 
 *       text     => 'Description', 
 *       selected => (TRUE|FALSE),
 *    ),
 *  ...
 * )
 *
 * @param string $name Name for select
 * @param array  $options hash with options
 * @return string html snippet
 */
function ib_make_html_select($name,$options) {
    $html = "<select name='$name' id='$name' size='1'>";
    foreach ($options as $value => $option) {
	$html .= "<option value='{$value}'";
	if (isset($option['selected']) and $option['selected'])
	    $html .= " selected='selected'";
	$html .= ">{$option['text']}</option>";
    }
    $html .= "</select>";

    return $html;
} // ib_make_html_select

/* here's where the action finally happens */

// remove magic quotes
ib_magic_quotes_remove($_GET);
ib_magic_quotes_remove($_POST);

// ---- how to sort the files?
// possible values
$ib_sortkey_allowed = array ('name', 'mtime'); 
$ib_sortorder_allowed = array('asc', 'desc');

// array for output
$ib_sortoptions = array(
    'mtime_desc' => array(
	'text' => 'last modification (newer first)'),
    'mtime_asc'	=> array(
	'text' => 'last modification (older first)'),
    'name_asc' => array(
	'text' => 'name (A-z)'),
    'name_desc'	=> array(
	'text' => 'name (z-A)'),
    );

// these are the defaults
$ib_sortkey   = 'mtime';
$ib_sortorder = 'desc';

if (isset($_REQUEST['sortby'])) { // GET or POST
    list ($key, $order) = explode('_',$_REQUEST['sortby']);
    if (in_array($key,$ib_sortkey_allowed))
	$ib_sortkey = $key;
    if (in_array($order,$ib_sortorder_allowed))
	$ib_sortorder = $order;
}

$ib_sortby = $ib_sortkey . '_' . $ib_sortorder;
$ib_sortoptions[$ib_sortby]['selected'] = TRUE;

// ---- if multiple pages, which page do you want?
$ib_page = 1; // the default

if (isset($_REQUEST['page']) and is_numeric($_REQUEST['page'])) {
    $ib_page = floor($_REQUEST['page']);
}

$imgdesc = '';
if (isset($_POST['imgdesc'])) {
    $imgdesc = str_replace('"', '&quot;', $_POST['imgdesc']);
    $imgdesc = str_replace("'", '&#8217;', $imgdesc);
    $imgdesc = htmlspecialchars($imgdesc);
}


// ----( Step 1: Select image )-------------------------------------------------

// ---- "Sort by: ..." selector
    // -> 'sortby' = ( see $ib_sortoptions )
    print "<div id='step'>"
	. "<form action='{$_SERVER['PHP_SELF']}' method='get'><div>"
	. "Sorted by: "	. ib_make_html_select('sortby',$ib_sortoptions)
	. "<input type='submit' value='Resort' />"
	. "</div></form></div>\n"
	. "<div id='ibwrapper'>";

// ---- get contents of the upload directory
    $ib_files = ib_get_uploaded_files();
    if (empty($ib_files)) {
	print "<p>Sorry, I can't find any files in your upload directory :-(</p>\n"
       . "<p>Try <a href='upload.php'>uploading some photos</a>.</p>\n"
	    . "<p><a href='#' onclick='window.close()'>Close window</a></p>\n";
	return FALSE;
    }

    // sort files
    if (FALSE === ($ib_files = ib_sort_hash_by_secondary_value($ib_files, $ib_sortkey, $ib_sortorder))) {
		print "<p>Error sorting by $ib_sortkey :-(</p>\n";
		return FALSE;
    }

// ---- navigation for pages of images
    $ib_pages = ceil(count($ib_files)/$ib_images_per_page);
    if ($ib_page > $ib_pages) $ib_page = $ib_pages;

    $ib_pagelinks = array();
    for ($count = 1; $count <= $ib_pages; $count++) {
		if ($ib_page == $count) {
	    $ib_pagelinks[] = "<strong>$count</strong>";
		} else {
	    $ib_pagelinks[] = "<a href='{$_SERVER['PHP_SELF']}?page={$count}"
			. "&amp;sortby={$ib_sortby}'>$count</a>";
		}
    }
    print '<p>Page: ' . implode('&nbsp;- ', $ib_pagelinks) . "</p>\n";

// ---- display images for selection
    reset($ib_files);
    for ($count = 1; $count <= count($ib_files); $count++) {
		list ($file, $meta) = each($ib_files);
	
		if (!(($ib_page-1)*$ib_images_per_page < $count
	      and $count <= $ib_page*$ib_images_per_page))
	    continue; // this image is not part of the requested page
	
		$html = '';
	
	// description and display image
		if (isset($meta['thumb'])) { // thumb
	   	$html .= "<div class='image'><div class='title'><strong>" 
			. htmlspecialchars($file) . "</strong><br />"
			. " (";
		}
	   if (isset($meta['width']) and isset($meta['height'])) {
			$html .= "{$meta['width']} x {$meta['height']} px, ";
		   $html .= round($meta['filesize']/1024,2) . ' kB)'
	 		. "</div>\n" . ib_make_html_img($meta['thumb']);
		}
		$html .= "\n<form action='{$_SERVER['PHP_SELF']}' method='post'>"
	    	. "<div class='select'>"
            // 'Add it!' button
	    	. "<input type='button' value='Add thumbnail'"
	    	. " onclick=\"targetopener('";
		$html .= ib_make_code_single_image(FALSE, $meta);
		$html .= "', 'true')\" /> <strong>OR</strong> ";
		$html .= "<input type='button' value='Add full image'"
			. " onclick=\"targetopener('";
	   $html .= ib_make_code_single_image($meta, $meta);
		$html .= "', 'true')\" />";
	   $html .= "<input type='hidden' name='file' value='". urlencode($file) ."' />"
	    	. "<input type='hidden' name='sortby' value='{$ib_sortby}' />"
	    	. "<input type='hidden' name='page' value='{$ib_page}' />"
	    	. "<input type='hidden' name='step' value='2' />"
	    	. "</div></form></div>\n";
	 print $html;
    }
// ---- footer
    print '<p>Page: ' . implode('&nbsp;- ', $ib_pagelinks) . "</p>\n";
    print "<p><a href='#' onclick='window.close()'>Close window</a></p>\n";

// ====( output footer )========================================================
?>
</div><!-- ibwrapper -->
</body>
</html>
<?php // ex:set tabstop=3: ?>
